@@ -1,6 +1,7 @@
 /*
- * rm_once.c    2002/08      kmatsui
- *  to remove '#pragma __once' line from the top of header files
+ * rm_once.c    2002/08, 2004/11    kmatsui
+ *  to remove '#pragma __once' and '#pragma once' line from the top of
+ *  header files
  */
 
 #include    "stdio.h"
@@ -10,7 +11,8 @@
 main( int argc, char ** argv)
 {
     char    buf[ BUFSIZ];
-    char    *once = "\n#pragma __once\n\n";
+    char    *__once = "\n#pragma __once\n\n";
+    char    *once = "\n#pragma once\n\n";
     char    *tmp = "tmp.once";
     char    *fname;
     FILE    *fp_in, *fp_out;
@@ -21,7 +23,7 @@ main( int argc, char ** argv)
         if ((fp_in = fopen( *argv, "r")) != NULL) {
             for (cp = buf, i = 0; i < 3; i++, cp += strlen( cp))
                 fgets( cp, BUFSIZ, fp_in);
-            if (strcmp( buf, once) != 0) {
+            if (strcmp( buf, once) != 0 || strcmp( buf, __once) != 0) {
                 fprintf( stderr, "Skipped %s\n", *argv);
                 fclose( fp_in);
                 continue;
